/******************************************
eagle Projekt: adruino_bed
Roland Nickel - 02/2018
 ******************************************/
#include "DataIO_PCF8574.h"
#include <inttypes.h>
#include "Wire.h"
#include "Arduino.h"

/******************************************
 Init
 Addr ... 7Bit Adresse angeben
 ******************************************/
DataIO_PCF8574::DataIO_PCF8574(uint8_t iic_Addr)
{
	Wire.begin();
  _Addr = iic_Addr;
}

/******************************************
 Adresse rcklesen (reale Schreibadresse)
 ******************************************/
uint8_t DataIO_PCF8574::getAddr(void)
{
	return (_Addr << 1);
}

/******************************************
 Begin
 - Mode ist die erste Ausgabe auf den PCF
 - Alle Bit "1" sind als Eingang nutzbar
 - Bei Taster-PCF also "0xFF"
 - Bei LED-PCF sind bei 0xFF alle AUS
 ******************************************/
uint8_t DataIO_PCF8574::begin(uint8_t mode)
{
	uint8_t rt;
	_Data = mode;

	delayMicroseconds(100);

	Wire.beginTransmission(_Addr);
	Wire.write(_Data);
	rt = Wire.endTransmission();   

	delayMicroseconds(100);
	return rt;
}

/******************************************
 Byte auf den PCF ausgeben
 ******************************************/
uint8_t DataIO_PCF8574::writeData(uint8_t data)
{
	uint8_t rt;
	_Data = data;

	delayMicroseconds(100);

	Wire.beginTransmission(_Addr);
	Wire.write(_Data);
	rt = Wire.endTransmission();   

	delayMicroseconds(100);
	return rt;
}

/******************************************
 Byte vom PCF lesen
 ******************************************/
uint8_t DataIO_PCF8574::readData(void)
{
	uint8_t data=0;
	delayMicroseconds(100);

	Wire.requestFrom(_Addr, 1);							// request 1 bytes from slave
	data = Wire.read();    									// receive 1 byte
	
	delayMicroseconds(100);
	return data;
}

/******************************************
 Decode Ausgabe Ports fr LED
 ******************************************/
uint8_t DataIO_PCF8574::decodePort(uint8_t bit)
{
	uint8_t tmp = 0;
	switch (bit)
	{
		case 0:
			tmp = 0x10;
			break;
		case 1:
			tmp = 0x40;
			break;
		case 2:
			tmp = 0x20;
			break;
		case 3:
			tmp = 0x80;
			break;
		case 4:
			tmp = 0x08;
			break;
		case 5:
			tmp = 0x04;
			break;
		case 6:
			tmp = 0x02;
			break;
		default:
			tmp = 0x01;
			break;
	}
	return tmp;
}

/******************************************
 Set Bit X set Port auf "1"
 ******************************************/
uint8_t DataIO_PCF8574::setBit(uint8_t bit)
{
	uint8_t rt;	
	uint8_t tmp = decodePort(bit);
	_Data |= tmp;
	delayMicroseconds(100);
	Wire.beginTransmission(_Addr);
	Wire.write(_Data);
	rt = Wire.endTransmission();   
	delayMicroseconds(100);
	
	return rt;
}

/******************************************
 Clear Bit X set Port auf "0"
 ******************************************/
uint8_t DataIO_PCF8574::clrBit(uint8_t bit)
{
	uint8_t rt;	
	uint8_t tmp = ~decodePort(bit);
	_Data &= tmp;
	delayMicroseconds(100);
	Wire.beginTransmission(_Addr);
	Wire.write(_Data);
	rt = Wire.endTransmission();   
	delayMicroseconds(100);
	
	return rt;
}

/******************************************
 Clear LED X set Port auf "1"
 ******************************************/
uint8_t DataIO_PCF8574::ledOFF(uint8_t bit)
{
	uint8_t rt;	
	uint8_t tmp = decodePort(bit);
	_Data |= tmp;
	delayMicroseconds(100);
	Wire.beginTransmission(_Addr);
	Wire.write(_Data);
	rt = Wire.endTransmission();   
	delayMicroseconds(100);
	
	return rt;
}

/******************************************
 Set LED X set Port auf "0"
 ******************************************/
uint8_t DataIO_PCF8574::ledON(uint8_t bit)
{
	uint8_t rt;	
	uint8_t tmp = ~decodePort(bit);
	_Data &= tmp;
	delayMicroseconds(100);
	Wire.beginTransmission(_Addr);
	Wire.write(_Data);
	rt = Wire.endTransmission();   
	delayMicroseconds(100);
	
	return rt;
}

/******************************************
 Decode Taster
 ******************************************/
uint8_t DataIO_PCF8574::decodeTaster(uint8_t data, uint8_t bit)
{
	uint8_t tmp = 0;
	switch (bit)
	{
		case 0:
			tmp = 0x02;
			break;
		case 1:
			tmp = 0x01;
			break;
		case 2:
			tmp = 0x08;
			break;
		case 3:
			tmp = 0x04;
			break;
		case 4:
			tmp = 0x80;
			break;
		case 5:
			tmp = 0x40;
			break;
		case 6:
			tmp = 0x10;
			break;
		default:
			tmp = 0x20;
			break;
	}
	data &= tmp;
	if(data>0){
		return 0;
	}
	return 1;
}

/******************************************
 Taster X Status auslesen
 0 ... nicht bettigt
 1 ... bettigt
 ******************************************/
uint8_t DataIO_PCF8574::getTaster(uint8_t bit)
{
	uint8_t data;
	delayMicroseconds(100);

	Wire.requestFrom(_Addr, 1);    	// request 1 bytes from slave device #2
  data = Wire.read();    					// receive a byte

	data = decodeTaster(data, bit);
	delayMicroseconds(100);
	return data;
}

/******************************************
               E N D E
 ******************************************/
